@@ -351,7 +351,14 @@ static PyObject *Reader_metadata(PyObject *self, PyObject *UNUSED(args)) {
     }
 
     MMDB_entry_data_list_s *entry_data_list;
-    MMDB_get_metadata_as_entry_data_list(mmdb_obj->mmdb, &entry_data_list);
+    int status =
+        MMDB_get_metadata_as_entry_data_list(mmdb_obj->mmdb, &entry_data_list);
+    if (status != MMDB_SUCCESS) {
+        PyErr_Format(MaxMindDB_error,
+                     "Error decoding metadata. %s",
+                     MMDB_strerror(status));
+        return NULL;
+    }
     MMDB_entry_data_list_s *original_entry_data_list = entry_data_list;
 
     PyObject *metadata_dict = from_entry_data_list(&entry_data_list);
@@ -744,10 +751,7 @@ static PyObject *from_map(MMDB_entry_data_list_s **entry_data_list) {
     const uint32_t map_size = (*entry_data_list)->entry_data.data_size;
 
     uint32_t i;
-    // entry_data_list cannot start out NULL (see from_entry_data_list). We
-    // check it in the loop because it may become NULL.
-    // coverity[check_after_deref]
-    for (i = 0; i < map_size && entry_data_list; i++) {
+    for (i = 0; i < map_size && *entry_data_list; i++) {
         *entry_data_list = (*entry_data_list)->next;
 
         PyObject *key = PyUnicode_FromStringAndSize(
@@ -785,10 +789,7 @@ static PyObject *from_array(MMDB_entry_data_list_s **entry_data_list) {
     }
 
     uint32_t i;
-    // entry_data_list cannot start out NULL (see from_entry_data_list). We
-    // check it in the loop because it may become NULL.
-    // coverity[check_after_deref]
-    for (i = 0; i < size && entry_data_list; i++) {
+    for (i = 0; i < size && *entry_data_list; i++) {
         *entry_data_list = (*entry_data_list)->next;
         PyObject *value = from_entry_data_list(entry_data_list);
         if (value == NULL) {
