@@ -1,7 +1,8 @@
-"""Constants used in the API"""
+"""Constants used in the API."""
 
 MODE_AUTO = 0
 MODE_MMAP_EXT = 1
 MODE_MMAP = 2
 MODE_FILE = 4
 MODE_MEMORY = 8
+MODE_FD = 16
