@@ -19,30 +19,24 @@ use Test::MaxMind::DB::Common::Util qw( standard_test_metadata );
 my $Dir = dirname( abs_path($0) );
 
 sub main {
-    my @sizes = ( 24, 28, 32 );
+    my @sizes      = ( 24, 28, 32 );
     my @ipv4_range = ( '1.1.1.1', '1.1.1.32' );
 
     my @ipv4_subnets = Net::Works::Network->range_as_subnets(@ipv4_range);
     for my $record_size (@sizes) {
         write_test_db(
-            $record_size,
-            \@ipv4_subnets,
-            { ip_version => 4 },
+            $record_size, \@ipv4_subnets, { ip_version => 4 },
             'ipv4',
         );
     }
 
     write_broken_pointers_test_db(
-        24,
-        \@ipv4_subnets,
-        { ip_version => 4 },
+        24, \@ipv4_subnets, { ip_version => 4 },
         'broken-pointers',
     );
 
     write_broken_search_tree_db(
-        24,
-        \@ipv4_subnets,
-        { ip_version => 4 },
+        24, \@ipv4_subnets, { ip_version => 4 },
         'broken-search-tree',
     );
 
@@ -53,9 +47,7 @@ sub main {
 
     for my $record_size (@sizes) {
         write_test_db(
-            $record_size,
-            \@ipv6_subnets,
-            { ip_version => 6 },
+            $record_size, \@ipv6_subnets, { ip_version => 6 },
             'ipv6',
         );
 
@@ -168,8 +160,7 @@ sub write_test_db {
 
     my $filename = sprintf(
         "$Dir/MaxMind-DB-test-%s-%i.mmdb",
-        $ip_version_name,
-        $record_size,
+        $ip_version_name, $record_size,
     );
     open my $fh, '>', $filename;
 
@@ -243,13 +234,10 @@ sub write_test_db {
             ::4.5.6.7/128
             abcd::/64
             1000::1234:0000/112
-        );
+            );
 
         for my $subnet (@subnets) {
-            $writer->insert_network(
-                $subnet,
-                \%all_types,
-            );
+            $writer->insert_network( $subnet, \%all_types, );
         }
 
         $writer->insert_network(
@@ -379,13 +367,10 @@ sub write_test_db {
             ::4.5.6.7/128
             abcd::/64
             1000::1234:0000/112
-        );
+            );
 
         for my $subnet (@subnets) {
-            $writer->insert_network(
-                $subnet,
-                \%nested,
-            );
+            $writer->insert_network( $subnet, \%nested, );
         }
 
         open my $fh, '>', "$Dir/MaxMind-DB-test-nested.mmdb";
@@ -466,6 +451,7 @@ sub _universal_map_key_type_callback {
         country                        => 'map',
         domain                         => 'utf8_string',
         geoname_id                     => 'uint32',
+        ip_risk                        => 'double',
         ipv4_24                        => 'uint32',
         ipv4_32                        => 'uint32',
         ipv6_32                        => 'uint32',
@@ -487,6 +473,8 @@ sub _universal_map_key_type_callback {
         location                       => 'map',
         longitude                      => 'double',
         metro_code                     => 'uint16',
+        mobile_country_code            => 'utf8_string',
+        mobile_network_code            => 'utf8_string',
         names                          => 'map',
         organization                   => 'utf8_string',
         population_density             => 'uint32',
