@@ -1,5 +1,4 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
+from __future__ import annotations
 
 import ipaddress
 import multiprocessing
@@ -7,29 +6,31 @@ import os
 import pathlib
 import threading
 import unittest
-import unittest.mock as mock
-from multiprocessing import Process, Pipe
-from typing import Union, Type
+from typing import TYPE_CHECKING, cast
+from unittest import mock
 
 import maxminddb
 
 try:
     import maxminddb.extension
 except ImportError:
-    maxminddb.extension = None  # type: ignore
+    maxminddb.extension = None  # type: ignore[assignment]
 
-from maxminddb import open_database, InvalidDatabaseError
+from maxminddb import InvalidDatabaseError, open_database
 from maxminddb.const import (
     MODE_AUTO,
-    MODE_MMAP_EXT,
-    MODE_MMAP,
+    MODE_FD,
     MODE_FILE,
     MODE_MEMORY,
-    MODE_FD,
+    MODE_MMAP,
+    MODE_MMAP_EXT,
 )
 
+if TYPE_CHECKING:
+    from maxminddb.reader import Reader
 
-def get_reader_from_file_descriptor(filepath, mode):
+
+def get_reader_from_file_descriptor(filepath: str, mode: int) -> Reader:
     """Patches open_database() for class TestFDReader()."""
     if mode == MODE_FD:
         with open(filepath, "rb") as mmdb_fh:
@@ -41,10 +42,9 @@ def get_reader_from_file_descriptor(filepath, mode):
         return maxminddb.open_database(filepath, mode)
 
 
-class BaseTestReader(object):
-    readerClass: Union[
-        Type["maxminddb.extension.Reader"], Type["maxminddb.reader.Reader"]
-    ]
+class BaseTestReader(unittest.TestCase):
+    mode: int
+    reader_class: type[maxminddb.extension.Reader | maxminddb.reader.Reader]
     use_ip_objects = False
 
     # fork doesn't work on Windows and spawn would involve pickling the reader,
@@ -52,12 +52,12 @@ class BaseTestReader(object):
     if os.name != "nt":
         mp = multiprocessing.get_context("fork")
 
-    def ipf(self, ip):
+    def ipf(self, ip: str) -> ipaddress.IPv4Address | ipaddress.IPv6Address | str:
         if self.use_ip_objects:
             return ipaddress.ip_address(ip)
         return ip
 
-    def test_reader(self):
+    def test_reader(self) -> None:
         for record_size in [24, 28, 32]:
             for ip_version in [4, 6]:
                 file_name = (
@@ -77,7 +77,7 @@ class BaseTestReader(object):
                     self._check_ip_v6(reader, file_name)
                 reader.close()
 
-    def test_get_with_prefix_len(self):
+    def test_get_with_prefix_len(self) -> None:
         decoder_record = {
             "array": [1, 2, 3],
             "boolean": True,
@@ -175,23 +175,30 @@ class BaseTestReader(object):
 
         for test in tests:
             with open_database(
-                "tests/data/test-data/" + test["file_name"], self.mode
+                "tests/data/test-data/" + cast("str", test["file_name"]),
+                self.mode,
             ) as reader:
-                (record, prefix_len) = reader.get_with_prefix_len(test["ip"])
+                (record, prefix_len) = reader.get_with_prefix_len(
+                    cast("str", test["ip"]),
+                )
 
                 self.assertEqual(
                     prefix_len,
                     test["expected_prefix_len"],
-                    f"expected prefix_len of {test['expected_prefix_len']} for {test['ip']}"
-                    + f" in {test['file_name']} but got {prefix_len}",
+                    f"expected prefix_len of {test['expected_prefix_len']}"
+                    f" for {test['ip']}"
+                    f" in {test['file_name']} but got {prefix_len}",
                 )
                 self.assertEqual(
                     record,
                     test["expected_record"],
-                    "expected_record for " + test["ip"] + " in " + test["file_name"],
+                    "expected_record for "
+                    + cast("str", test["ip"])
+                    + " in "
+                    + cast("str", test["file_name"]),
                 )
 
-    def test_iterator(self):
+    def test_iterator(self) -> None:
         tests = (
             {
                 "database": "ipv4",
@@ -234,16 +241,20 @@ class BaseTestReader(object):
 
         for record_size in [24, 28, 32]:
             for test in tests:
-                f = f'tests/data/test-data/MaxMind-DB-test-{test["database"]}-{record_size}.mmdb'
+                f = (
+                    f"tests/data/test-data/MaxMind-DB-test-{test['database']}"
+                    f"-{record_size}.mmdb"
+                )
                 reader = open_database(f, self.mode)
                 networks = [str(n) for (n, _) in reader]
                 self.assertEqual(networks, test["expected"], f)
 
-    def test_decoder(self):
+    def test_decoder(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
-        record = reader.get(self.ipf("::1.1.1.0"))
+        record = cast("dict", reader.get(self.ipf("::1.1.1.0")))
 
         self.assertEqual(record["array"], [1, 2, 3])
         self.assertEqual(record["boolean"], True)
@@ -266,27 +277,30 @@ class BaseTestReader(object):
         self.assertEqual(1329227995784915872903807060280344576, record["uint128"])
         reader.close()
 
-    def test_metadata_pointers(self):
+    def test_metadata_pointers(self) -> None:
         with open_database(
-            "tests/data/test-data/MaxMind-DB-test-metadata-pointers.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-metadata-pointers.mmdb",
+            self.mode,
         ) as reader:
             self.assertEqual(
                 "Lots of pointers in metadata",
                 reader.metadata().database_type,
             )
 
-    def test_no_ipv4_search_tree(self):
+    def test_no_ipv4_search_tree(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-no-ipv4-search-tree.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-no-ipv4-search-tree.mmdb",
+            self.mode,
         )
 
         self.assertEqual(reader.get(self.ipf("1.1.1.1")), "::0/64")
         self.assertEqual(reader.get(self.ipf("192.1.1.1")), "::0/64")
         reader.close()
 
-    def test_ipv6_address_in_ipv4_database(self):
+    def test_ipv6_address_in_ipv4_database(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-ipv4-24.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-ipv4-24.mmdb",
+            self.mode,
         )
         with self.assertRaisesRegex(
             ValueError,
@@ -297,27 +311,29 @@ class BaseTestReader(object):
             reader.get(self.ipf("2001::"))
         reader.close()
 
-    def test_opening_path(self):
+    def test_opening_path(self) -> None:
         with open_database(
-            pathlib.Path("tests/data/test-data/MaxMind-DB-test-decoder.mmdb"), self.mode
+            pathlib.Path("tests/data/test-data/MaxMind-DB-test-decoder.mmdb"),
+            self.mode,
         ) as reader:
             self.assertEqual(reader.metadata().database_type, "MaxMind DB Decoder Test")
 
-    def test_no_extension_exception(self):
-        real_extension = maxminddb._extension
-        maxminddb._extension = None
+    def test_no_extension_exception(self) -> None:
+        real_extension = maxminddb._extension  # noqa: SLF001
+        maxminddb._extension = None  # type: ignore[assignment]  # noqa: SLF001
         with self.assertRaisesRegex(
             ValueError,
             "MODE_MMAP_EXT requires the maxminddb.extension module to be available",
         ):
             open_database(
-                "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", MODE_MMAP_EXT
+                "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+                MODE_MMAP_EXT,
             )
-        maxminddb._extension = real_extension
+        maxminddb._extension = real_extension  # noqa: SLF001
 
-    def test_broken_database(self):
+    def test_broken_database(self) -> None:
         reader = open_database(
-            "tests/data/test-data/" "GeoIP2-City-Test-Broken-Double-Format.mmdb",
+            "tests/data/test-data/GeoIP2-City-Test-Broken-Double-Format.mmdb",
             self.mode,
         )
         with self.assertRaisesRegex(
@@ -329,21 +345,23 @@ class BaseTestReader(object):
             reader.get(self.ipf("2001:220::"))
         reader.close()
 
-    def test_ip_validation(self):
+    def test_ip_validation(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         with self.assertRaisesRegex(
-            ValueError, "'not_ip' does not appear to be an IPv4 or " "IPv6 address"
+            ValueError,
+            "'not_ip' does not appear to be an IPv4 or IPv6 address",
         ):
             reader.get("not_ip")
         reader.close()
 
-    def test_missing_database(self):
+    def test_missing_database(self) -> None:
         with self.assertRaisesRegex(FileNotFoundError, "No such file or directory"):
             open_database("file-does-not-exist.mmdb", self.mode)
 
-    def test_nondatabase(self):
+    def test_nondatabase(self) -> None:
         with self.assertRaisesRegex(
             InvalidDatabaseError,
             r"Error opening database file \(README.rst\). "
@@ -352,7 +370,7 @@ class BaseTestReader(object):
             open_database("README.rst", self.mode)
 
     # This is from https://github.com/maxmind/MaxMind-DB-Reader-python/issues/58
-    def test_database_with_invalid_utf8_key(self):
+    def test_database_with_invalid_utf8_key(self) -> None:
         reader = open_database(
             "tests/data/bad-data/maxminddb-python/bad-unicode-in-map-key.mmdb",
             self.mode,
@@ -360,15 +378,15 @@ class BaseTestReader(object):
         with self.assertRaises(UnicodeDecodeError):
             reader.get_with_prefix_len("163.254.149.39")
 
-    def test_too_many_constructor_args(self):
+    def test_too_many_constructor_args(self) -> None:
         with self.assertRaises(TypeError):
-            self.readerClass("README.md", self.mode, 1)
+            self.reader_class("README.md", self.mode, 1)  # type: ignore[arg-type,call-arg]
 
-    def test_bad_constructor_mode(self):
+    def test_bad_constructor_mode(self) -> None:
         with self.assertRaisesRegex(ValueError, r"Unsupported open mode \(100\)"):
-            self.readerClass("README.md", mode=100)
+            self.reader_class("README.md", mode=100)  # type:  ignore[arg-type]
 
-    def test_no_constructor_args(self):
+    def test_no_constructor_args(self) -> None:
         with self.assertRaisesRegex(
             TypeError,
             r" 1 required positional argument|"
@@ -376,108 +394,120 @@ class BaseTestReader(object):
             r"takes at least 2 arguments|"
             r"function missing required argument \'database\' \(pos 1\)",
         ):
-            self.readerClass()
+            self.reader_class()  # type:  ignore[call-arg]
 
-    def test_too_many_get_args(self):
+    def test_too_many_get_args(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         with self.assertRaises(TypeError):
-            reader.get(self.ipf("1.1.1.1"), "blah")
+            reader.get(self.ipf("1.1.1.1"), "blah")  # type:  ignore[call-arg]
         reader.close()
 
-    def test_no_get_args(self):
+    def test_no_get_args(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         with self.assertRaises(TypeError):
-            reader.get()
+            reader.get()  # type:  ignore[call-arg]
         reader.close()
 
-    def test_incorrect_get_arg_type(self):
+    def test_incorrect_get_arg_type(self) -> None:
         reader = open_database("tests/data/test-data/GeoIP2-City-Test.mmdb", self.mode)
         with self.assertRaisesRegex(
-            TypeError, "argument 1 must be a string or ipaddress object"
+            TypeError,
+            "argument 1 must be a string or ipaddress object",
         ):
-            reader.get(1)
+            reader.get(1)  # type:  ignore[arg-type]
         reader.close()
 
-    def test_metadata_args(self):
+    def test_metadata_args(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         with self.assertRaises(TypeError):
-            reader.metadata("blah")
+            reader.metadata("blah")  # type:  ignore[call-arg]
         reader.close()
 
-    def test_metadata_unknown_attribute(self):
+    def test_metadata_unknown_attribute(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         metadata = reader.metadata()
         with self.assertRaisesRegex(
-            AttributeError, "'Metadata' object has no " "attribute 'blah'"
+            AttributeError,
+            "'Metadata' object has no attribute 'blah'",
         ):
-            metadata.blah
+            metadata.blah  # type:  ignore[attr-defined]  # noqa: B018
         reader.close()
 
-    def test_close(self):
+    def test_close(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         reader.close()
 
-    def test_double_close(self):
+    def test_double_close(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         reader.close()
-        self.assertIsNone(reader.close(), "Double close does not throw an exception")
+        # Check that calling close again doesn't raise an exception
+        reader.close()
 
-    def test_closed_get(self):
+    def test_closed_get(self) -> None:
         if self.mode in [MODE_MEMORY, MODE_FD]:
             return
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         reader.close()
         with self.assertRaisesRegex(
-            ValueError, "Attempt to read from a closed MaxMind DB.|closed"
+            ValueError,
+            "Attempt to read from a closed MaxMind DB.|closed",
         ):
             reader.get(self.ipf("1.1.1.1"))
 
-    def test_with_statement(self):
+    def test_with_statement(self) -> None:
         filename = "tests/data/test-data/MaxMind-DB-test-ipv4-24.mmdb"
         with open_database(filename, self.mode) as reader:
             self._check_ip_v4(reader, filename)
         self.assertEqual(reader.closed, True)
 
-    def test_with_statement_close(self):
+    def test_with_statement_close(self) -> None:
         filename = "tests/data/test-data/MaxMind-DB-test-ipv4-24.mmdb"
         reader = open_database(filename, self.mode)
         reader.close()
 
-        with self.assertRaisesRegex(
-            ValueError, "Attempt to reopen a closed MaxMind DB"
+        with (
+            self.assertRaisesRegex(
+                ValueError,
+                "Attempt to reopen a closed MaxMind DB",
+            ),
+            reader,
         ):
-            with reader:
-                pass
+            pass
 
-    def test_closed(self):
+    def test_closed(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         self.assertEqual(reader.closed, False)
         reader.close()
         self.assertEqual(reader.closed, True)
 
-    # XXX - Figure out whether we want to have the same behavior on both the
-    #       extension and the pure Python reader. If we do, the pure Python
-    #       reader will need to throw an exception or the extension will need
-    #       to keep the metadata in memory.
-    def test_closed_metadata(self):
+    def test_closed_metadata(self) -> None:
         reader = open_database(
-            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb", self.mode
+            "tests/data/test-data/MaxMind-DB-test-decoder.mmdb",
+            self.mode,
         )
         reader.close()
 
@@ -485,7 +515,7 @@ class BaseTestReader(object):
         # segfault
         try:
             metadata = reader.metadata()
-        except IOError as ex:
+        except OSError as ex:
             self.assertEqual(
                 "Attempt to read from a closed MaxMind DB.",
                 str(ex),
@@ -502,25 +532,26 @@ class BaseTestReader(object):
         def test_threading(self):
             self._check_concurrency(threading.Thread)
 
-        def _check_concurrency(self, worker_class):
+        def _check_concurrency(self, worker_class) -> None:  # noqa: ANN001
             reader = open_database(
-                "tests/data/test-data/GeoIP2-Domain-Test.mmdb", self.mode
+                "tests/data/test-data/GeoIP2-Domain-Test.mmdb",
+                self.mode,
             )
 
-            def lookup(pipe):
+            def lookup(pipe) -> None:  # noqa: ANN001
                 try:
                     for i in range(32):
                         reader.get(self.ipf(f"65.115.240.{i}"))
                     pipe.send(1)
-                except:
+                except:  # noqa: E722
                     pipe.send(0)
                 finally:
-                    if worker_class is self.mp.Process:
+                    if worker_class is self.mp.Process:  # type: ignore[attr-defined]
                         reader.close()
                     pipe.close()
 
             pipes = [self.mp.Pipe() for _ in range(32)]
-            procs = [worker_class(target=lookup, args=(c,)) for (p, c) in pipes]
+            procs = [worker_class(target=lookup, args=(c,)) for (_, c) in pipes]
             for proc in procs:
                 proc.start()
             for proc in procs:
@@ -528,11 +559,16 @@ class BaseTestReader(object):
 
             reader.close()
 
-            count = sum([p.recv() for (p, c) in pipes])
+            count = sum([p.recv() for (p, _) in pipes])
 
             self.assertEqual(count, 32, "expected number of successful lookups")
 
-    def _check_metadata(self, reader, ip_version, record_size):
+    def _check_metadata(
+        self,
+        reader: Reader,
+        ip_version: int,
+        record_size: int,
+    ) -> None:
         metadata = reader.metadata()
 
         self.assertEqual(2, metadata.binary_format_major_version, "major version")
@@ -541,7 +577,8 @@ class BaseTestReader(object):
         self.assertEqual(metadata.database_type, "Test")
 
         self.assertEqual(
-            {"en": "Test Database", "zh": "Test Database Chinese"}, metadata.description
+            {"en": "Test Database", "zh": "Test Database Chinese"},
+            metadata.description,
         )
         self.assertEqual(metadata.ip_version, ip_version)
         self.assertEqual(metadata.languages, ["en", "zh"])
@@ -549,7 +586,7 @@ class BaseTestReader(object):
 
         self.assertEqual(metadata.record_size, record_size)
 
-    def _check_ip_v4(self, reader, file_name):
+    def _check_ip_v4(self, reader: Reader, file_name: str) -> None:
         for i in range(6):
             address = "1.1.1." + str(pow(2, i))
             self.assertEqual(
@@ -579,7 +616,7 @@ class BaseTestReader(object):
         for ip in ["1.1.1.33", "255.254.253.123"]:
             self.assertIsNone(reader.get(self.ipf(ip)))
 
-    def _check_ip_v6(self, reader, file_name):
+    def _check_ip_v6(self, reader: Reader, file_name: str) -> None:
         subnets = ["::1:ffff:ffff", "::2:0:0", "::2:0:40", "::2:0:50", "::2:0:58"]
 
         for address in subnets:
@@ -611,83 +648,87 @@ class BaseTestReader(object):
             self.assertIsNone(reader.get(self.ipf(ip)))
 
 
-def has_maxminddb_extension():
-    return maxminddb.extension and hasattr(maxminddb.extension, "Reader")
+def has_maxminddb_extension() -> bool:
+    return maxminddb.extension is not None and hasattr(
+        maxminddb.extension,
+        "Reader",
+    )
 
 
 @unittest.skipIf(
     not has_maxminddb_extension() and not os.environ.get("MM_FORCE_EXT_TESTS"),
     "No C extension module found. Skipping tests",
 )
-class TestExtensionReader(BaseTestReader, unittest.TestCase):
+class TestExtensionReader(BaseTestReader):
     mode = MODE_MMAP_EXT
 
     if has_maxminddb_extension():
-        readerClass = maxminddb.extension.Reader
+        reader_class = maxminddb.extension.Reader
 
 
 @unittest.skipIf(
     not has_maxminddb_extension() and not os.environ.get("MM_FORCE_EXT_TESTS"),
     "No C extension module found. Skipping tests",
 )
-class TestExtensionReaderWithIPObjects(BaseTestReader, unittest.TestCase):
+class TestExtensionReaderWithIPObjects(BaseTestReader):
     mode = MODE_MMAP_EXT
     use_ip_objects = True
 
     if has_maxminddb_extension():
-        readerClass = maxminddb.extension.Reader
+        reader_class = maxminddb.extension.Reader
 
 
-class TestAutoReader(BaseTestReader, unittest.TestCase):
+class TestAutoReader(BaseTestReader):
     mode = MODE_AUTO
 
-    readerClass: Union[
-        Type["maxminddb.extension.Reader"], Type["maxminddb.reader.Reader"]
-    ]
+    reader_class: type[maxminddb.extension.Reader | maxminddb.reader.Reader]
     if has_maxminddb_extension():
-        readerClass = maxminddb.extension.Reader
+        reader_class = maxminddb.extension.Reader
     else:
-        readerClass = maxminddb.reader.Reader
+        reader_class = maxminddb.reader.Reader
 
 
-class TestMMAPReader(BaseTestReader, unittest.TestCase):
+class TestMMAPReader(BaseTestReader):
     mode = MODE_MMAP
-    readerClass = maxminddb.reader.Reader
+    reader_class = maxminddb.reader.Reader
 
 
 # We want one pure Python test to use IP objects, it doesn't
 # really matter which one.
-class TestMMAPReaderWithIPObjects(BaseTestReader, unittest.TestCase):
+class TestMMAPReaderWithIPObjects(BaseTestReader):
     mode = MODE_MMAP
     use_ip_objects = True
-    readerClass = maxminddb.reader.Reader
+    reader_class = maxminddb.reader.Reader
 
 
-class TestFileReader(BaseTestReader, unittest.TestCase):
+class TestFileReader(BaseTestReader):
     mode = MODE_FILE
-    readerClass = maxminddb.reader.Reader
+    reader_class = maxminddb.reader.Reader
 
 
-class TestMemoryReader(BaseTestReader, unittest.TestCase):
+class TestMemoryReader(BaseTestReader):
     mode = MODE_MEMORY
-    readerClass = maxminddb.reader.Reader
+    reader_class = maxminddb.reader.Reader
 
 
-class TestFDReader(BaseTestReader, unittest.TestCase):
-    def setUp(self):
+class TestFDReader(BaseTestReader):
+    def setUp(self) -> None:
         self.open_database_patcher = mock.patch(__name__ + ".open_database")
         self.addCleanup(self.open_database_patcher.stop)
         self.open_database = self.open_database_patcher.start()
         self.open_database.side_effect = get_reader_from_file_descriptor
 
     mode = MODE_FD
-    readerClass = maxminddb.reader.Reader
+    reader_class = maxminddb.reader.Reader
 
 
 class TestOldReader(unittest.TestCase):
-    def test_old_reader(self):
+    def test_old_reader(self) -> None:
         reader = maxminddb.Reader("tests/data/test-data/MaxMind-DB-test-decoder.mmdb")
-        record = reader.get("::1.1.1.0")
+        record = cast("dict", reader.get("::1.1.1.0"))
 
         self.assertEqual(record["array"], [1, 2, 3])
         reader.close()
+
+
+del BaseTestReader
